// CS559-2007 - Project 1 - Picture Processor
//
// Example Solution, by Mike Gleicher
// begin 9/10/2007

// file: PictureProcessor.H
// implements the basic parsing for the little scripting language
// implements the things in the PictureProcessor header file
// also has a few example commands for testing
#include "PictureProcessor.H"
#include "TargaImage.H"

#include <stdio.h>

// the global variable that keeps the head of the list
Command* commands=0;

// the generic command knows how to install itself on the list of commands
Command::Command(const char* _verb, int _nargs)
: verb(_verb), nargs(_nargs)
{
	// add to the beginning of the linked list
	next = commands;
	commands = this;
}

// break a string into words (for easy parsing)
void breakString(char* str, std::vector<const char*>& words)
{
	// start with no words
	words.clear();

	// scan through the string, starting at the beginning
	char* p = str;

	// stop when we hit the end of the string
	while(*p) {
		// skip over leading whitespace - stop at the first character or end of string
		while (*p && *p<=' ') p++;

		// now we're pointing at the first thing after the spaces
		// make sure its not a comment, and that we're not at the end of the string
		// (that's actually the same thing)
		if (! (*p) || *p == '#')
			break;

		// so we're pointing at a word! add it to the word list
		words.push_back(p);

		// now find the end of the word
		while(*p > ' ') p++;	// stop at space or end of string

		// if its ethe end of the string, we're done
		if (! *p) break;

		// otherwise, turn this space into and end of string (to end the word)
		// and keep going
		*p = 0;
		p++;
	}
}

Context::Context() : lineNo(0), variables(26)
{
	// make sure the variables are empty (set to null)
	for(int i=0; i<26; i++)
		variables[i] = 0;
}
Context::~Context()
{
	// delete anything in the variables
	for(int i=0; i<26; i++)
		setVar(i,0);
}
TargaImage* Context::getVar(const int id) const
{
	if ( (id<0) || (id>25) ) {
		printf("Error (line %d): variable access out of bounds\n",lineNo);
		return 0;
	} else
		return variables[id];
}
void Context::setVar(const int id, TargaImage* img)
{
	if ( (id<0) || (id>25) ) {
		printf("Error (line %d): variable access out of bounds\n",lineNo);
	} else {
		// remember - if we set the variable, we first need to delete any image
		// that was held there before!
		if (variables[id])
			delete variables[id];
		variables[id] = img;
	}
}

int Context::variableIndex(const char *v) const
{
	if (!v) {
		printf("Error (line %d): no variable name!\n",lineNo);
		return -1;
	}
	if (strlen(v) != 1) {
		printf("Error (line %d): variable must have 1 character name!\n",lineNo);
		return -1;
	}
	if ((v[0] >= 'A') && (v[0] <= 'Z')) return v[0]-'A';
	if ((v[0] >= 'a') && (v[0] <= 'z')) return v[0]-'a';

	printf("Error (line %d): variable must have 1 character name!\n",lineNo);
	return -1;
}

TargaImage* Context::getImage(const char* v) const
{
	int var = variableIndex(v);

	if (v<0) return 0;
	return getVar(var);
}

//
// an example command for testing
class TestCmd : public Command
{
public:
	TestCmd() : Command("test",1) {};
    virtual int doit(const std::vector<const char*>&,
		             Context&) {
		printf("Test command!\n");
		return 0;
	}
};
// note that we need to create a (single) global instance of this
// when it is created, it will be installed
TestCmd testCmd;

// another example command for testing - prints info about an image
class InfoCmd : public Command
{
public:
	InfoCmd() : Command("info",2) {};
    virtual int doit(const std::vector<const char*>& cmd,
		             Context& context) {
		TargaImage* img = context.getImage(cmd[1]);
		if (img)
			printf("Image size %d %d\n",img->width(),img->height());
		else
			printf("No Image\n");
		return 0;
	};
};
InfoCmd infoCmd;